import sys
import re

# -----------------------------------------------
# Read log files for stdout and stderr
# we write them separately to avoid parallel
# intermix of the two stream, but we allow
# stdout on rank 0 to intermix with other stdout
# -----------------------------------------------
par_out = open(sys.argv[1]).read()
par_err = open(sys.argv[2]).read()
log = par_out + par_err

# -----------------------------------------------
# Figure out the number of processors
# Because of a mpi.barrier(), we know
# that output will be flushed before
# we start on other output on stdout
# -----------------------------------------------
pattern = re.compile(r'0 SIZE=([0-9]+)')
match = pattern.search(log)
assert match is not None
nprocs = int(match.group(1))

# -----------------------------------------------
# Recreate the log in a serialized order
# -----------------------------------------------
out = ''
err = ''

# Record processor count
out += '0 SIZE=%d\n'%nprocs

# stdout/stderr to console (English)
for rank in range(nprocs):
    out += '%d hello\n'%rank
    err += '%d goodbye\n'%rank

# stdout/stderr to /dev/null (Hungarian)
out += "%d Jo'napot\n"%0
err += '%d Viszontltsra\n'%0

# stdout/stderr to activated processor one at
# a time (SHOUTED ENGLISH)
ordered_out = ''
ordered_err = ''
for rank in range(nprocs):
    ordered_out += '%d HELLO\n'%rank
    ordered_err += '%d GOODBYE\n'%rank
out += ordered_out
err += ordered_err

# synchronizedWrite() (Texan)
sync = ''
for rank in range(nprocs):
    sync += '%d howdy \n'%rank
out += sync

# stdout/stderr to queue files [Spanish](rank 0 still outputs)
out += '%d hola\n'%0
err += '%d adios\n'%0

# -----------------------------------------------
# Compare the full contents of the serialized log
# to the actual log.  Sorting allows us to
# ignore intermixing in the output
# -----------------------------------------------
serial_log = [c for c in (out+err)]
real_log = [c for c in log]

serial_log.sort()
real_log.sort()

assert ''.join(serial_log) == ''.join(real_log)


# -----------------------------------------------
# Compare the queued out and err files
# -----------------------------------------------
for rank in range(1,nprocs):
    Qout = open('output.out.%d'%rank).read()
    assert Qout == '%d hola\n'%rank,rank

    Qerr = open('errput.err.%d'%rank).read()
    assert Qerr == '%d adios\n'%rank,rank

# -----------------------------------------------
# Make sure the ordered write pieces are present
# in proper order
# -----------------------------------------------
assert out.find(ordered_out) > 0
assert err.find(ordered_err) > 0
assert out.find(sync) > 0
