#**************************************************************************#
#* FILE   **************      scaling_tests.py     ************************#
#**************************************************************************#
#* Author: Patrick Miller August  7 2003                                  *#
#**************************************************************************#


import mpi
from pyMPI.pingpong import *
import random

TRIALS=2 # TODO: set back to 100

def safediv(x,y):
    try:
        return x/y
    except ZeroDivisionError:
        return 0*x

def safe_plot(fp,comm_size,t0,t1,K0):
    if comm.rank == 0:
        K = safediv(t0,t1)
        if K0 is None: K0 = K
        predict = K0*t1
        err = safediv(t0-K0*t1,t0)*100
        if err < 0.0: err = 0.0
        fp.write('%d\t%g\t%g\t%g\t%g\t%4.1f\n'%(
            comm_size,t0,t1,K,predict,err))

    return K0


def py_integrate(comm,n):
    h = 1.0/n
    sum = 0.0
    for i in range(comm.rank+1,n+1,comm.size):
        x = h * (i-0.5)
        sum = sum + 4.0/(1.0+x*x)

    myAnswer = h * sum
    answer = comm.reduce(myAnswer,mpi.SUM)
    return answer

msg = "The quick brown fox jumps over the lazy dog"
def ping(comm,s,r,msg):
    if comm.rank == s:
        comm.send(msg,r)
        comm.recv(r)
    else:
        msg,stat = comm.recv(s)
        comm.send(msg,s)
    return

if __name__ == '__main__':
    comms = [ mpi.WORLD.comm_create(range(comm_size+1)) for comm_size in range(mpi.size)]

    # -----------------------------------------------
    #  calcpi
    # -----------------------------------------------
    K0 = None # Assigned later
    if mpi.rank == 0: print "calcpi"
    fp = open('calcpi.plt','w')
    fp.write('# calcpi\n')
    fp.write('# n\tPython\t\tC\t\tratio\tprediction\t%error\n')
    for comm in comms:
        if comm is None: continue

        # constant work per processor
        rectangles = 10000

        comm.barrier()
        t0,py_pi = timefunction(TRIALS,py_integrate,comm,rectangles)
        t1,c_pi =  timefunction(TRIALS,c_integrate,comm,rectangles)

        K0 = safe_plot(fp,comm.size,t0,t1,K0)


    # -----------------------------------------------
    #  barrier
    # -----------------------------------------------
    K0 = None
    mpi.barrier()
    if mpi.rank == 0: print "barrier"
    fp = open('barrier.plt','w')
    fp.write('# barrier\n')
    fp.write('# n\tPython\t\tC\t\tratio\tprediction\t%error\n')
    for comm in comms:
        if comm is None: continue

        comm.barrier()
        t0,py = timefunction(TRIALS,comm.barrier)
        t1,c  = timefunction(TRIALS,c_barrier,comm)

        K0 = safe_plot(fp,comm.size,t0,t1,K0)

    # -----------------------------------------------
    #  areduce
    # -----------------------------------------------
    K0 = None
    mpi.barrier()
    if mpi.rank == 0: print "areduce"
    fp = open('areduce.plt','w')
    fp.write('# areduce\n')
    fp.write('# n\tPython\t\tC\t\tratio\tprediction\t%error\n')
    for comm in comms:
        if comm is None: continue

        comm.barrier()
        t0,py = timefunction(TRIALS,comm.allreduce,3.0,mpi.SUM,type=float)
        t1,c  = timefunction(TRIALS,allreduce_doublesum,3.0,comm)

        K0 = safe_plot(fp,comm.size,t0,t1,K0)

    # -----------------------------------------------
    #  ping_neighbor
    # -----------------------------------------------
    K0 = None
    mpi.barrier()
    if mpi.rank == 0: print "ping_neighbor"
    fp = open('ping_neighbor.plt','w')
    fp.write('# ping_neighbor\n')
    fp.write('# n\tPython\t\tC\t\tratio\tprediction\t%error\n')
    for comm in comms:
        if comm is None: continue
        if comm.size % 2: continue

        s = (comm.rank/2)*2
        r = s+1

        comm.barrier()
        t0,py = timefunction(TRIALS,ping,comm,s,r,msg)
        t1,c  = timefunction(TRIALS,c_ping,comm,s,r,msg)

        K0 = safe_plot(fp,comm.size,t0,t1,K0)


    # -----------------------------------------------
    #  ping_random
    # -----------------------------------------------
    K0 = None
    mpi.barrier()
    if mpi.rank == 0: print "ping_random"
    fp = open('ping_random.plt','w')
    fp.write('# ping_random\n')
    fp.write('# n\tPython\t\tC\t\tratio\tprediction\t%error\n')
    for comm in comms:
        if comm is None: continue
        if comm.size % 2: continue

        processors = range(comm.size)
        random.shuffle(processors)
        processors = comm.bcast(processors)
        senders = processors[:comm.size/2]
        receivers = processors[comm.size/2:]
        if comm.rank in senders:
            s = comm.rank
            r = receivers[senders.index(comm.rank)]
        else:
            s = senders[receivers.index(comm.rank)]
            r = comm.rank


        comm.barrier()
        t0,py = timefunction(TRIALS,ping,comm,s,r,msg)
        t1,c  = timefunction(TRIALS,c_ping,comm,s,r,msg)

        K0 = safe_plot(fp,comm.size,t0,t1,K0)

    # -----------------------------------------------
    #  sr_neighbor
    # -----------------------------------------------
    K0 = None
    mpi.barrier()
    if mpi.rank == 0: print "sr_neighbor"
    fp = open('sr_neighbor.plt','w')
    fp.write('# sr_neighbor\n')
    fp.write('# n\tPython\t\tC\t\tratio\tprediction\t%error\n')
    for comm in comms:
        if comm is None: continue
        if comm.size % 2: continue

        s = (comm.rank/2)*2
        r = s+1

        comm.barrier()
        if comm.rank == s:
            t0,py = timefunction(TRIALS,comm.sendrecv,msg,r)
        else:
            t0,py = timefunction(TRIALS,comm.sendrecv,msg,s)
        t1,c  = timefunction(TRIALS,c_sr,comm,s,r,msg)

        K0 = safe_plot(fp,comm.size,t0,t1,K0)


    # -----------------------------------------------
    #  sr_random
    # -----------------------------------------------
    K0 = None
    mpi.barrier()
    if mpi.rank == 0: print "sr_random"
    fp = open('sr_random.plt','w')
    fp.write('# sr_random\n')
    fp.write('# n\tPython\t\tC\t\tratio\tprediction\t%error\n')
    for comm in comms:
        if comm is None: continue
        if comm.size % 2: continue

        processors = range(comm.size)
        random.shuffle(processors)
        processors = comm.bcast(processors)
        senders = processors[:comm.size/2]
        receivers = processors[comm.size/2:]
        if comm.rank in senders:
            s = comm.rank
            r = receivers[senders.index(comm.rank)]
        else:
            s = senders[receivers.index(comm.rank)]
            r = comm.rank

        comm.barrier()
        if comm.rank == s:
            t0,py = timefunction(TRIALS,comm.sendrecv,msg,r)
        else:
            t0,py = timefunction(TRIALS,comm.sendrecv,msg,s)
        t1,c  = timefunction(TRIALS,c_sr,comm,s,r,msg)

        K0 = safe_plot(fp,comm.size,t0,t1,K0)

    # -----------------------------------------------
    #  gather
    # -----------------------------------------------
    mpi.barrier()
    K0 = None
    if mpi.rank == 0: print "gather"
    fp = open('gather.plt','w')
    fp.write('# gather\n')
    fp.write('# n\tPython\t\tC\t\tratio\tprediction\t%error\n')
    for comm in comms:
        if comm is None: continue

        comm.barrier()
        t0,py = timefunction(TRIALS,comm.gather,[mpi.rank])
        t1,c  = timefunction(TRIALS,c_gather,comm,mpi.rank)

        K0 = safe_plot(fp,comm.size,t0,t1,K0)



    # -----------------------------------------------
    #  bcast
    # -----------------------------------------------
    mpi.barrier()
    K0 = None
    if mpi.rank == 0: print "bcast"
    fp = open('bcast.plt','w')
    fp.write('# bcast\n')
    fp.write('# n\tPython\t\tC\t\tratio\tprediction\t%error\n')
    for comm in comms:
        if comm is None: continue

        comm.barrier()
        t0,py = timefunction(TRIALS,comm.bcast,mpi.rank)
        t1,c  = timefunction(TRIALS,c_bcast,comm,mpi.rank)

        K0 = safe_plot(fp,comm.size,t0,t1,K0)

