import mpi
from numpy.oldnumeric import ones
from pyMPI.stencil import threepoint

ZONES = 10
INITIAL_TEMPERATURE = 100
STEPS = 1

# Set the field to the initial temperature
T = ones(ZONES,'d')*INITIAL_TEMPERATURE

# define some sources
sources = (
    (0,200),
    (ZONES-1,0),
    )

# -----------------------------------------------
# Scatter zones across processors
# -----------------------------------------------
T = mpi.scatter(T)
ZONES = len(T)
assert ZONES > 0

# -----------------------------------------------
# Adjust the source locations
# -----------------------------------------------
high_zone = mpi.scan(ZONES,mpi.SUM)
low_zone = high_zone - ZONES

def global_to_local(z):
    lz = z - low_zone
    if lz < 0 or lz > ZONES:
        return None
    return lz

def local_to_global(z):
    return z + low_zone

# How to apply the sources
def set_sources(sources,field):
    for position,temp in sources:
        lz = global_to_local(position)
        if lz is not None: field[lz] = temp
    return

# Deal with the boundaries
BC_left = INITIAL_TEMPERATURE
BC_right = INITIAL_TEMPERATURE

# Take some steps
set_sources(sources,T)
for i in range(STEPS):

    # Get boundary conditions from adjoining process
    if mpi.size == 1:
        pass
    elif mpi.rank == 0:
        BC_right,status = mpi.recv(1)
        mpi.send(T[-1],1)
    elif mpi.rank == mpi.size -1:
        BC_left,status = mpi.sendrecv(T[0],mpi.rank-1,mpi.rank-1)
    else:
        BC_right,status = mpi.sendrecv(T[0],mpi.rank-1,mpi.rank+1)
        BC_left,status = mpi.sendrecv(T[-1],mpi.rank+1,mpi.rank-1)

    # Take a step
    threepoint(BC_left,T,BC_right)

    # Apply the sources
    set_sources(sources,T)

T = mpi.gather(T)
if mpi.rank == 0:
    print T
